// console.cpp: the console buffer, its display, and command line control

//*** All edited lines/functions shown below for use with CureControl:

hashtable<int, keym> keyms(512); //*** Expand existing key hashtable to allow room for controller support

const char *getkeyname(int code)
{
    keym *km = keyms.access(code);
    return km ? km->name : "NULL";
}
ICOMMAND(0, getkeyname, "i", (int *code), {result(getkeyname(*code));}); //*** for menus, configured and debugged (game crashed with old code)

void searchbindlist(const char *action, int type, int limit, const char *sep, const char *pretty, const char *cap, vector<char> &names) //*** "cap" added for "excluded keys with said prefix" for use in bindings in Options menu (ex: "JOY" - any key mapped with the frefix "JOY" will be skipped):
{
    const char *name1 = NULL, *name2 = NULL, *lastname = NULL;
    int found = 0;
    enumerate(keyms, keym, km,
    {
        int caplen = strlen(cap);
        if(caplen > 0)
        {
            if(!strncmp(km.name, cap, caplen)) break;
        }
        char *act = ((!km.actions[type] || !*km.actions[type]) && type ? km.actions[keym::ACTION_DEFAULT] : km.actions[type]);
        if(!strcmp(act, action))
        {
            if(!name1) name1 = km.name;
            else if(!name2) name2 = km.name;
            else
            {
                if(lastname)
                {
                    ADDSEP(',', "");
                    names.put(lastname, strlen(lastname));
                }
                else
                {
                    names.put(name1, strlen(name1));
                    ADDSEP(',', "");
                    names.put(name2, strlen(name2));
                }
                lastname = km.name;
            }
            ++found;
            if(limit > 0 && found >= limit) break;
        }
    });
    if(lastname)
    {
        ADDSEP(',', sep);
        names.put(lastname, strlen(lastname));
    }
    else
    {
        if(name1) names.put(name1, strlen(name1));
        if(name2)
        {
            ADDSEP('\0', sep);
            names.put(name2, strlen(name2));
        }
    }
    names.add('\0');
}
//*** More use of "cap" here: (also update iengine.h!)
ICOMMAND(0, searchbinds,     "sisss", (char *action, int *limit, char *sep, char *pretty, char *cap), { vector<char> list; searchbindlist(action, keym::ACTION_DEFAULT, max(*limit, 0), sep, pretty, cap, list); result(list.getbuf()); });
ICOMMAND(0, searchspecbinds, "sisss", (char *action, int *limit, char *sep, char *pretty, char *cap), { vector<char> list; searchbindlist(action, keym::ACTION_SPECTATOR, max(*limit, 0), sep, pretty, cap, list); result(list.getbuf()); });
ICOMMAND(0, searcheditbinds, "sisss", (char *action, int *limit, char *sep, char *pretty, char *cap), { vector<char> list; searchbindlist(action, keym::ACTION_EDITING, max(*limit, 0), sep, pretty, cap, list); result(list.getbuf()); });
ICOMMAND(0, searchwaitbinds, "sisss", (char *action, int *limit, char *sep, char *pretty, char *cap), { vector<char> list; searchbindlist(action, keym::ACTION_WAITING, max(*limit, 0), sep, pretty, cap, list); result(list.getbuf()); });
