// main.cpp: initialisation & main loop

//*** Joystick mod insertions below included in two functions:

#include "joystick.h"

void checkinput()
{
    SDL_Event event;
    int lasttype = 0, lastbut = 0;
    while(events.length() || SDL_PollEvent(&event))
    {
        if(events.length()) event = events.remove(0);

        switch (event.type)
        {
            case SDL_QUIT:
                quit();
                break;

#if !defined(WIN32) && !defined(__APPLE__)
            case SDL_VIDEORESIZE:
                screenres(&event.resize.w, &event.resize.h);
                break;
#endif

            case SDL_KEYDOWN:
            case SDL_KEYUP:
                keypress(event.key.keysym.sym, event.key.state==SDL_PRESSED, uni2cube(event.key.keysym.unicode));
                break;

            case SDL_ACTIVEEVENT:
            {
                if(event.active.state & SDL_APPINPUTFOCUS)
                {
                    activewindow = event.active.gain ? true : false;

                    if(autograbinput)
                        setvar("grabinput", event.active.gain ? 1 : 0, true);
                }
                if(event.active.state & SDL_APPACTIVE)
                    minimized = !event.active.gain;
                break;
            }
            case SDL_MOUSEMOTION:
            {
                if((grabinput || activewindow) && !skipmousemotion(event, true))
                {
                    //conoutf("mouse: %d %d, %d %d [%s, %d]", event.motion.xrel, event.motion.yrel, event.motion.x, event.motion.y, warpmouse ? "true" : "false", ignoremouse);
                    int dx = event.motion.xrel, dy = event.motion.yrel;
                    checkmousemotion(dx, dy);
                    if(game::mousemove(dx, dy, event.motion.x, event.motion.y, screen->w, screen->h))
                        resetcursor(true, false); // game controls engine cursor
                }
                break;
            }
            case SDL_MOUSEBUTTONDOWN:
            case SDL_MOUSEBUTTONUP:
            {
                if(lasttype==event.type && lastbut==event.button.button) break; // why?? get event twice without it
                keypress(-event.button.button, event.button.state!=0, 0);
                lasttype = event.type;
                lastbut = event.button.button;
                break;
            }
        }
        joystick::process_event(&event);
    }
    joystick::move();
}

int main(int argc, char **argv)
{
    #ifdef WIN32
    //atexit((void (__cdecl *)(void))_CrtDumpMemoryLeaks);
    #ifndef _DEBUG
    #ifndef __GNUC__
    __try {
    #endif
    #endif
    #endif

    setlogfile(NULL);
    setlocations(true);

    char *initscript = NULL;
    initing = INIT_RESET;
    bool hashome = findarg(argc, argv, "-h"), hasinit = findarg(argc, argv, "-r");
    if(!hashome && !hasinit)
    {
        execfile("init.cfg", false);
        restoredinits = hasinit = true;
    }
    for(int i = 1; i<argc; i++)
    {
        if(argv[i][0]=='-') switch(argv[i][1])
        {
            case 'r': execfile(argv[i][2] ? &argv[i][2] : "init.cfg", false); restoredinits = hasinit = true; break;
            case 'd':
            {
                switch(argv[i][2])
                {
                    case 'w': scr_w = clamp(atoi(&argv[i][3]), SCR_MINW, SCR_MAXW); if(!findarg(argc, argv, "-dh")) scr_h = -1; break;
                    case 'h': scr_h = clamp(atoi(&argv[i][3]), SCR_MINH, SCR_MAXH); if(!findarg(argc, argv, "-dw")) scr_w = -1; break;
                    case 'd': depthbits = atoi(&argv[i][3]); break;
                    case 'c': colorbits = atoi(&argv[i][3]); break;
                    case 'a': fsaa = atoi(&argv[i][3]); break;
                    case 'v': vsync = atoi(&argv[i][3]); break;
                    case 'f': fullscreen = atoi(&argv[i][3]); break;
                    case 's': stencilbits = atoi(&argv[i][3]); break;
                    case 'u':
                    {
                        extern int useshaders, shaderprecision, forceglsl;
                        int n = atoi(&argv[i][3]);
                        useshaders = n > 0 ? 1 : 0;
                        shaderprecision = clamp(n >= 4 ? n - 4 : n - 1, 0, 2);
                        forceglsl = n >= 4 ? 1 : 0;
                        break;
                    }
                    default: conoutf("\frunknown display option %c", argv[i][2]); break;
                }
                break;
            }
            case 'x': initscript = &argv[i][2]; break;
            default:
                if(!serveroption(argv[i])) gameargs.add(argv[i]);
                else if(argv[i][1] == 'h' && !hasinit)
                {
                    execfile("init.cfg", false);
                    restoredinits = hasinit = true;
                }
                break;
        }
        else gameargs.add(argv[i]);
    }

    initing = NOT_INITING;

    conoutf("loading enet..");
    if(enet_initialize()<0) fatal("Unable to initialise network module");
    atexit(enet_deinitialize);
    enet_time_set(0);

    conoutf("loading game..");
    initgame();

	joystick::init();

    conoutf("loading sdl..");
    int par = 0;
    #ifdef _DEBUG
    par = SDL_INIT_NOPARACHUTE;
    #ifdef WIN32
    SetEnvironmentVariable("SDL_DEBUG", "1");
    #endif
    #endif

    //#ifdef WIN32
    //SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
    //#endif

    par |= SDL_INIT_TIMER|SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE;
    if(SDL_Init(par) < 0) fatal("Unable to initialize SDL: %s", SDL_GetError());
    ignoremouse += 3;

    conoutf("loading video mode..");
    const SDL_VideoInfo *video = SDL_GetVideoInfo();
    if(video)
    {
        desktopw = video->current_w;
        desktoph = video->current_h;
    }
    int usedcolorbits = 0, useddepthbits = 0, usedfsaa = 0;
    setupscreen(usedcolorbits, useddepthbits, usedfsaa);

    conoutf("loading video misc..");
    ncursor = SDL_GetCursor();
    showcursor(false);
    setscreensaver(false);
    keyrepeat(false);
    setcaption("please wait..");

    signal(SIGINT, fatalsignal);
    signal(SIGILL, fatalsignal);
    signal(SIGABRT, fatalsignal);
    signal(SIGFPE, fatalsignal);
    signal(SIGSEGV, fatalsignal);
    signal(SIGTERM, fatalsignal);
#if !defined(WIN32) && !defined(__APPLE__)
    signal(SIGHUP, reloadsignal);
    signal(SIGQUIT, fatalsignal);
    signal(SIGKILL, fatalsignal);
    signal(SIGPIPE, fatalsignal);
    signal(SIGALRM, fatalsignal);
#endif

    conoutf("loading gl..");
    gl_checkextensions();
    gl_init(scr_w, scr_h, usedcolorbits, useddepthbits, usedfsaa);
    if(!(notexture = textureload("gametex/shared/notexture")) ||
        !(blanktexture = textureload("gametex/shared/blank")))
        fatal("could not find core textures");

    conoutf("loading sound..");
    initsound();

    conoutf("loading defaults..");

    if(!execfile("stdlib.cfg", false)) fatal("cannot find data files");
    if(!setfont("default")) fatal("no default font specified");
    inbetweenframes = true;
    progress(0, "please wait..");

    conoutf("loading gl effects..");
    progress(0, "loading gl effects..");
    loadshaders();

    conoutf("loading world..");
    progress(0, "loading world..");
    emptymap(0, true, NULL, true);

    conoutf("loading config..");
    progress(0, "loading config..");
    rehash(false);
    smartmusic(true, false);

    conoutf("loading required data..");
    progress(0, "loading required data..");
    preloadtextures();
    particleinit();
    initdecals();

    trytofindocta();
    conoutf("loading main..");
    progress(0, "loading main..");
    if(initscript) execute(initscript, true);

#ifdef WIN32
    SDL_SysWMinfo wminfo;
    SDL_VERSION(&wminfo.version);
    if(SDL_GetWMInfo(&wminfo)) ShowWindow(wminfo.window, SW_SHOW);
#endif
    if(autograbinput) setvar("grabinput", 1, true);

    localconnect(false);
    resetfps();
    UI::setup();
    //game::preload();

    for(int frameloops = 0; ; frameloops = frameloops >= INT_MAX-1 ? MAXFPSHISTORY+1 : frameloops+1)
    {
        int elapsed = updatetimer(true);
        updatefps(frameloops, elapsed);
        checkinput();
        menuprocess();

        if(frameloops)
        {
            RUNWORLD("on_update");
            game::updateworld();
        }

        checksleep(lastmillis);
        serverslice();
#ifdef IRC
        ircslice();
#endif
        if(frameloops)
        {
            game::recomputecamera(screen->w, screen->h);
            hud::update(screen->w, screen->h);
            setviewcell(camera1->o);
            updatetextures();
            updateparticles();
            updatesounds();
            UI::update();
            if(!minimized)
            {
                inbetweenframes = renderedframe = false;
                lastprogress = 0;
                gl_drawframe(screen->w, screen->h);
                renderedframe = true;
                swapbuffers();
                inbetweenframes = true;
            }
            defformatstring(cap)("%s - %s", game::gametitle(), game::gametext());
            setcaption(cap);
        }
    }

    ASSERT(0);
    return EXIT_FAILURE;

#if defined(WIN32) && !defined(_DEBUG) && !defined(__GNUC__)
    } __except(stackdumper(0, GetExceptionInformation()), EXCEPTION_CONTINUE_SEARCH) { return 0; }
#endif
}
